! module phase_space_pre
! 	implicit none
! 	contains
! !-----------------------------------------
! 	double precision function lambda(x,y)
! 	double precision,intent(in):: x,y

! 	lambda=1.d0+x**2+y**2-2.d0*x*y-2.d0*x-2.d0*y

! 	end function

! end module phase_space_pre

!-----------------------------------------
module phase_space
	! use phase_space_pre
	use SM_const
	implicit none
	contains
!-----------------------------------------
	double precision function lambda(x,y)
	double precision,intent(in):: x,y

	lambda=1.d0+x**2+y**2-2.d0*x*y-2.d0*x-2.d0*y

	end function
!-----------------------------------------
	! decay for phi -> VV* 
	! x should be like mV/mh

	double precision function F3body(x)
	double precision,intent(in):: x

	F3body=-ABS(1.d0-x**2)*(47.d0*x**2/2.d0-13.d0/2.d0+1.d0/x**2)&
      +3.d0*(1.d0-6.d0*x**2+4.d0*x**4)*ABS(LOG(x))+3.d0*&
      (1.d0-8.d0*x**2+20.d0*x**4)/sqrt(4.d0*x**2-1.d0)*&
      ACOS((3.d0*x**2-1.d0)/(2.d0*x**3))
	end function F3body

!-----------------------------------------
	! decay for phi -> phi' V*
	! The order of x and y is important! 
	! x should be m_phi'^2/m_phi^2, while y should be m_V^2/m_phi^2
	double precision function G3body(x,y)
	double precision,intent(in):: x,y

	G3body=1.d0/(12.d0*y)*(&
      2.d0*(-1.d0+x)**3&
     -9.d0*(-1.d0+x**2)*y&
     +6.d0*(-1.d0+x)*y**2&
     +6.d0*(1.d0+x-y)*y*sqrt(-lambda(x,y))&
     *(&
     atan(&
     (-1.d0+x-y)/sqrt(-lambda(x,y))&
     )&
     +atan(&
     (-1.d0+x+y)/sqrt(-lambda(x,y))&
     )&
     )&
     -3.d0*(&
      1.d0+(x-y)**2-2.d0*y&
     )*y*Log(x)&
     )

	end function G3body
!-----------------------------------------
	! decay for phi -> VV'*
	! The order of x and y is important! 
	! x should be m_V^2/m_phi^2, while y should be m_V'^2/m_phi^2
	double precision function H3body(x,y)
	double precision,intent(in):: x,y

	H3body=(atan(&
             (1.d0-x+y)/sqrt(-lambda(x,y))&
             )&
             +&
              atan(&
             (1.d0-x-y)/sqrt(-lambda(x,y))&
             )&
             )/&
             (&
             4.d0*x*sqrt(-lambda(x,y))&
             )*&
             (&
             -3.d0*x**3+(9.d0*y+7.d0)*x**2&
             -5.d0*(1.d0-y)**2*x+(1.d0-y)**3&
              )&
             +1.d0/(24.d0*x*y)*&
             (&
             (-1.d0+x)*&
             (6.d0*y**2+y*(39.d0*x-9.d0)+2.d0*(1.d0-x)**2)&
             -3.d0*y*&
             (y**2+2.d0*y*(3.d0*x-1.d0)-x*(3.d0*x+4.d0)+1.d0)*log(x)&
             )

	end function H3body
!-----------------------------------------
	! decay for phi -> VV'*
	! The order of x and y is important! 
	! x should be m_V^2/m_phi^2, while y should be m_V'^2/m_phi^2
	double precision function H33body(x,y)
	double precision,intent(in):: x,y

	H33body=(atan(&
             (1.d0-x)*sqrt(-lambda(x,y))/(y*(1+x)-(1-x)**2)&
             )&
              + 3.14159265359d0)/&
             (&
             4.d0*x*sqrt(-lambda(x,y))&
             )*&
             (&
             -3.d0*x**3+(9.d0*y+7.d0)*x**2&
             -5.d0*(1.d0-y)**2*x+(1.d0-y)**3&
              )&
             +1.d0/(24.d0*x*y)*&
             (&
             (-1.d0+x)*&
             (6.d0*y**2+y*(39.d0*x-9.d0)+2.d0*(1.d0-x)**2)&
             -3.d0*y*&
             (y**2+2.d0*y*(3.d0*x-1.d0)-x*(3.d0*x+4.d0)+1.d0)*log(x)&
             )

	end function H33body
!-----------------------------------------
	! decay for phi -> phi' V*
	double precision function SSpV3body(mphi,mphip,mV)
	double precision,intent(in):: mphi,mphip,mV
	integer:: i,j,div_width
	double precision:: x1,x2,kV,kS,dSSpV3body,gamV

	kS = (mphip/mphi)**2
	kV = (mV/mphi)**2
	if(mV .eq. mW) then
		gamV=(2.085d0/mphi)**2
	else if(mV .eq. mZ) then
		gamV=(2.4952d0/mphi)**2
	else
		gamV=0.d0
	endif

	div_width=100; SSpV3body=0.d0

	do i = 0, div_width
		dSSpV3body=0.d0
		x2 = dble(i)*(1.d0-kS)/dble(div_width)
		do j = 0, div_width
			x1 = 1.d0-x2-kS+(1.d0-kS/(1.d0-x2))*x2/dble(div_width)*dble(j)
			if(j==0 .or. j==div_width) then
				dSSpV3body = dSSpV3body&
								 +((1.d0-x1)*(1.d0-x2)-kS)/((1.d0-x1-x2-kS+kV)**2+kV*gamV)
			else if(mod(j,2)==1) then
				dSSpV3body = dSSpV3body&
								 +4.d0*((1.d0-x1)*(1.d0-x2)-kS)/((1.d0-x1-x2-kS+kV)**2+kV*gamV)
			else if(mod(j,2)==0) then
				dSSpV3body = dSSpV3body&
								 +2.d0*((1.d0-x1)*(1.d0-x2)-kS)/((1.d0-x1-x2-kS+kV)**2+kV*gamV)
			endif
		enddo
		dSSpV3body = dSSpV3body*(1.d0-kS/(1.d0-x2))*x2/dble(div_width)/3.d0
		if(i==0 .or. i==div_width) then
			SSpV3body = SSpV3body+dSSpV3body
		else if(mod(i,2)==1) then
			SSpV3body = SSpV3body+4.d0*dSSpV3body
		else if(mod(i,2)==0) then
			SSpV3body = SSpV3body+2.d0*dSSpV3body
		endif
	enddo
	SSpV3body = SSpV3body*(1.d0-kS)/dble(div_width)/3.d0

	end function SSpV3body
end module phase_space