module real_photon_emission
	use SM_const
	use func_forQCDco

	implicit none
	contains

!=====================================================
!     Bremsstrahlung functions
!=====================================================
	complex (kind(0d0)) function llambda(x,y,z)
	double precision, intent(in):: x,y,z

	llambda = x**2+y**2+z**2-2.d0*x*y-2.d0*y*z-2.d0*z*x

	end function
!--------------------------------------------  
	complex (kind(0d0)) function beta0(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	beta0 = (m0**2-m1**2-m2**2+sqrt(llambda(m0**2,m1**2,m2**2)))/(2.d0*m1*m2)

	end function
!--------------------------------------------  
	complex (kind(0d0)) function beta1(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	if(m1/=0.d0 .and. m2 /= 0.d0 .and. m1 == m2) then
	!		beta1 = (m0**2-m1**2+m2**2-sqrt(llambda(m0**2,m1**2,m2**2)))/(2.d0*m0*m2)
	!KS  in case of m1 == m2, we express beta1 by beta2 and beta0
		beta1 = 1.d0/beta2(m0,m1,m2)/beta0(m0,m1,m2)
	else if(m1/=0.d0 .and. m2 /= 0.d0 .and. m1 > m2) then
		beta1 = 1.d0/beta2(m0,m1,m2)/beta0(m0,m1,m2)
	else if(m1/=0.d0 .and. m2 /= 0.d0 .and. m2 > m1) then
		beta1 = (m0**2-m1**2+m2**2-sqrt(llambda(m0**2,m1**2,m2**2)))/(2.d0*m0*m2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		beta1 = m2/m0
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function beta2(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	if(m1/=0.d0 .and. m2 /= 0.d0 .and. m1 == m2) then
		beta2 = (m0**2+m1**2-m2**2-sqrt(llambda(m0**2,m1**2,m2**2)))/(2.d0*m0*m1)
	else if(m1/=0.d0 .and. m2 /= 0.d0 .and. m1 > m2) then
		beta2 = (m0**2+m1**2-m2**2-sqrt(llambda(m0**2,m1**2,m2**2)))/(2.d0*m0*m1)
	else if(m1/=0.d0 .and. m2 /= 0.d0 .and. m2 > m1) then
		beta2 = 1.d0/beta1(m0,m1,m2)/beta0(m0,m1,m2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		beta2 = 0.d0
	endif

	end function
!--------------------------------------------  
	complex (kind(0d0)) function Ibrems_00(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda,lamIR,getlambda

	lambda = llambda(m0**2,m1**2,m2**2)
	if(mphoton /= 0.d0) then
		lamIR = mphoton
	else if(mphoton == 0.d0) then
		lamIR = sqrt(getlambda())
	endif

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems_00 = (sqrt(lambda)*log(lambda/(lamIR*m0*m1*m2))-sqrt(lambda)&
	 		-(m1**2-m2**2)*log(beta1(m0,m1,m2)/beta2(m0,m1,m2))-m0**2*log(beta0(m0,m1,m2)))&
			/(4.d0*m0**4)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems_00 = (sqrt(lambda)*log(sqrt(lambda)/(lamIR*m0))-sqrt(lambda)&
			+m2**2*log(m2**2/m0**2))/(4.d0*m0**4)
	endif

	end function
!--------------------------------------------  
	complex (kind(0d0)) function Ibrems_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda,lamIR,getlambda

	lambda = llambda(m0**2,m1**2,m2**2)
	if(mphoton /= 0.d0) then
		lamIR = mphoton
	else if(mphoton == 0.d0) then
		lamIR = sqrt(getlambda())
	endif

	Ibrems_11 = (sqrt(lambda)*log(lambda/(lamIR*m0*m1*m2))-sqrt(lambda)&
		-(m0**2-m2**2)*log(beta0(m0,m1,m2)/beta2(m0,m1,m2))-m1**2*log(beta1(m0,m1,m2)))&
	 /(4.d0*m1**2*m0**2)

	end function
!--------------------------------------------  
	complex (kind(0d0)) function Ibrems_22(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda,lamIR,getlambda

	lambda = llambda(m0**2,m1**2,m2**2)
	if(mphoton /= 0.d0) then
		lamIR = mphoton
	else if(mphoton == 0.d0) then
		lamIR = sqrt(getlambda())
	endif

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems_22 = (sqrt(lambda)*log(lambda/(lamIR*m0*m1*m2))-sqrt(lambda)&
			-(m0**2-m1**2)*log(beta0(m0,m1,m2)/beta1(m0,m1,m2))-m2**2*log(beta2(m0,m1,m2)))&
	 		/(4.d0*m2**2*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems_22 = (sqrt(lambda)*log(sqrt(lambda)/(lamIR*m0))-sqrt(lambda)&
		+(m0**2+m2**2)*log(m2/m0))/(4.d0*m2**2*m0**2)
	endif

	end function
!--------------------------------------------  
	complex (kind(0d0)) function Ibrems_01(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda,lamIR
#include "looptools.h"

	lambda = llambda(m0**2,m1**2,m2**2)
	if(mphoton /= 0.d0) then
		lamIR = mphoton
	else if(mphoton == 0.d0) then
		lamIR = sqrt(getlambda())
	endif

	Ibrems_01 = (2.d0*log(lambda/(lamIR*m0*m1*m2))*log(beta2(m0,m1,m2))&
		+2.d0*log(beta2(m0,m1,m2))**2-log(beta0(m0,m1,m2))**2-log(beta1(m0,m1,m2))**2&
		+2.d0*Li2(1.d0-beta2(m0,m1,m2)**2)-Li2(1.d0-beta0(m0,m1,m2)**2)-Li2(1.d0-beta1(m0,m1,m2)**2))&
		/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems_02(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda,lamIR
#include "looptools.h"

	lambda = llambda(m0**2,m1**2,m2**2)
	if(mphoton /= 0.d0) then
		lamIR = mphoton
	else if(mphoton == 0.d0) then
		lamIR = sqrt(getlambda())
	endif

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems_02 = (2.d0*log(lambda/(lamIR*m0*m1*m2))*log(beta1(m0,m1,m2))&
			+2.d0*log(beta1(m0,m1,m2))**2-log(beta0(m0,m1,m2))**2-log(beta2(m0,m1,m2))**2&
			+2.d0*Li2(1.d0-beta1(m0,m1,m2)**2)-Li2(1.d0-beta0(m0,m1,m2)**2)-Li2(1.d0-beta2(m0,m1,m2)**2))&
			/(4.d0*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems_02 = (2.d0*log(sqrt(lambda)/(lamIR*m0))*log(m2/m0)&
			+log(m2/m0)**2+2.d0*Li2(1.d0-beta1(m0,0.d0,m2)**2))/(4.d0*m0**2)
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems_12(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	 Ibrems_12 = -Ibrems_01(m0,m1,m2)-Ibrems_02(m0,m1,m2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems = (sqrt(lambda)/2.d0*(m0**2+m1**2+m2**2)+2.d0*m0**2*m1**2*log(beta2(m0,m1,m2))&
			+2.d0*m0**2*m2**2*log(beta1(m0,m1,m2))+2.d0*m1**2*m2**2*log(beta0(m0,m1,m2)))/(4.d0*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems = (sqrt(lambda)/2.d0*(m0**2+m2**2)+2.d0*m0**2*m2**2*log(m2/m0))/(4.d0*m0**2)
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems_0(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems_0 = (-2.d0*m1**2*log(beta2(m0,m1,m2))-2.d0*m2**2*log(beta1(m0,m1,m2))&
			-sqrt(lambda))/(4.d0*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems_0 = -m2**2/(2.d0*m0**2)*log(m2/m0)-(m0**2-m2**2)/(4.d0*m0**2)
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems_1(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems_1 = (-2.d0*m0**2*log(beta2(m0,m1,m2))-2.d0*m2**2*log(beta0(m0,m1,m2))&
		-sqrt(lambda))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems_2(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems_2 = (-2.d0*m0**2*log(beta1(m0,m1,m2))-2.d0*m1**2*log(beta0(m0,m1,m2))&
			-sqrt(lambda))/(4.d0*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems_2 = (-2.d0*m0**2*log(m2/m0)-sqrt(lambda))/(4.d0*m0**2)
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems1_0(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems1_0 = (m1**4*log(beta2(m0,m1,m2))&
		-m2**2*(2.d0*m0**2-2.d0*m1**2+m2**2)*log(beta1(m0,m1,m2))&
		-sqrt(lambda)/4.d0*(m0**2-3.d0*m1**2+5.d0*m2**2))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems2_0(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems2_0 = (m2**4*log(beta1(m0,m1,m2))&
		-m1**2*(2.d0*m0**2-2.d0*m2**2+m1**2)*log(beta2(m0,m1,m2))&
		-sqrt(lambda)/4.d0*(m0**2-3.d0*m2**2+5.d0*m1**2))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems0_1(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems0_1 = (m0**4*log(beta2(m0,m1,m2))&
		-m2**2*(2.d0*m1**2-2.d0*m0**2+m2**2)*log(beta0(m0,m1,m2))&
		-sqrt(lambda)/4.d0*(m1**2-3.d0*m0**2+5.d0*m2**2))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems0_2(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	if(m1/=0.d0 .and. m2 /= 0.d0) then
		Ibrems0_2 = (m0**4*log(beta1(m0,m1,m2))&
			-m1**2*(2.d0*m2**2-2.d0*m0**2+m1**2)*log(beta0(m0,m1,m2))&
			-sqrt(lambda)/4.d0*(m2**2-3.d0*m0**2+5.d0*m1**2))/(4.d0*m0**2)
	else if(m1 == 0.d0 .and. m2 /= 0.d0) then
		Ibrems0_2 = (m0**4*log(m2/m0)-sqrt(lambda)/4.d0*(m2**2-3.d0*m0**2))/(4.d0*m0**2)
	endif

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems1_2(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	Ibrems1_2 = -Ibrems(m0,m1,m2)-Ibrems0_2(m0,m1,m2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems2_1(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	Ibrems2_1 = -Ibrems(m0,m1,m2)-Ibrems0_1(m0,m1,m2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems12_00(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	Ibrems12_00 = -(m1**4*log(beta2(m0,m1,m2))+m2**4*log(beta1(m0,m1,m2))&
		+sqrt(lambda)**3/(6.d0*m0**2)&
		+sqrt(lambda)/4.d0*(3.d0*m1**2+3.d0*m2**2-m0**2))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems02_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	Ibrems02_11 = -(m0**4*log(beta2(m0,m1,m2))+m2**4*log(beta0(m0,m1,m2))&
		+sqrt(lambda)**3/(6.d0*m1**2)&
		+sqrt(lambda)/4.d0*(3.d0*m0**2+3.d0*m2**2-m1**2))/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems00_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	Ibrems00_11 = -Ibrems0_1(m0,m1,m2)-Ibrems02_11(m0,m1,m2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems11_00(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	Ibrems11_00 = -Ibrems1_0(m0,m1,m2)-Ibrems12_00(m0,m1,m2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems22_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems22_11 = (2.d0*m0**2*(m0**2+m1**2-m2**2)*log(beta2(m0,m1,m2))&
		+sqrt(lambda)**3/(6.d0*m1**2)+2.d0*sqrt(lambda)*m0**2)/(4.d0*m0**2)

	end function
!--------------------------------------------
	complex (kind(0d0)) function Ibrems11_22(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2
	double precision:: lambda

	lambda = llambda(m0**2,m1**2,m2**2)

	Ibrems11_22 = (2.d0*m0**2*(m0**2+m2**2-m1**2)*log(beta1(m0,m1,m2))&
		+sqrt(lambda)**3/(6.d0*m2**2)+2.d0*sqrt(lambda)*m0**2)/(4.d0*m0**2)

	end function
!----------------------------------------------------------------------
!   QED functions for phi(p0,m0) -> f(p1,m1)f'(p2,m2)gam(q,mphoton)
!----------------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_00(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_00=&
		-4.d0*m0**2*Ibrems_0(m0,m1,m2)&
		-4.d0*m0**2*(m0**2-m1**2-m2**2)*Ibrems_00(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_11=&
		-2.d0*Ibrems(m0,m1,m2)&
		+2.d0*(m0**2+m1**2-m2**2)*Ibrems_1(m0,m1,m2)&
		-4.d0*m1**2*(m0**2-m1**2-m2**2)*Ibrems_11(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_22(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_22=&
		-2.d0*Ibrems(m0,m1,m2)&
		+2.d0*(m0**2-m1**2+m2**2)*Ibrems_2(m0,m1,m2)&
		-4.d0*m2**2*(m0**2-m1**2-m2**2)*Ibrems_22(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_01(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_01=&
		-2.d0*Ibrems0_1(m0,m1,m2)&
		-2.d0*(3.d0*m0**2-m1**2-3.d0*m2**2)*Ibrems_1(m0,m1,m2)&
		+4.d0*(m1**2+m2**2)*Ibrems_0(m0,m1,m2)&
		-4.d0*(m0**2-m1**2-m2**2)*(m0**2+m1**2-m2**2)*Ibrems_01(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_02(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_02=&
		+2.d0*Ibrems0_2(m0,m1,m2)&
		+2.d0*(3.d0*m0**2-3.d0*m1**2-m2**2)*Ibrems_2(m0,m1,m2)&
		-4.d0*(m1**2+m2**2)*Ibrems_0(m0,m1,m2)&
		+4.d0*(m0**2-m1**2-m2**2)*(m0**2-m1**2+m2**2)*Ibrems_02(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLL_12(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLL_12=&
		+8.d0*Ibrems(m0,m1,m2)&
		+2.d0*Ibrems2_1(m0,m1,m2)+2.d0*Ibrems1_2(m0,m1,m2)&
		-2.d0*(3.d0*m0**2-m1**2-3.d0*m2**2)*Ibrems_1(m0,m1,m2)&
		-2.d0*(3.d0*m0**2-3.d0*m1**2-m2**2)*Ibrems_2(m0,m1,m2)&
		+4.d0*(m0**2-m1**2-m2**2)**2*Ibrems_12(m0,m1,m2)
	
  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_00(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_00= 8.d0*m0**2*m1*m2*Ibrems_00(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_11(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_11= 8.d0*m1**3*m2*Ibrems_11(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_22(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_22= 8.d0*m1*m2**3*Ibrems_22(m0,m1,m2)

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_01(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_01= 8.d0*m1*m2*(Ibrems_0(m0,m1,m2)+Ibrems_1(m0,m1,m2)&
		+(m0**2+m1**2-m2**2)*Ibrems_01(m0,m1,m2))

  end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_02(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_02= -8.d0*m1*m2*(Ibrems_0(m0,m1,m2)+Ibrems_2(m0,m1,m2)&
		+(m0**2-m1**2+m2**2)*Ibrems_02(m0,m1,m2))

	end function
!-------------------------------------------------------------
	complex (kind(0d0)) function	OmegaLR_12(m0,m1,m2)
	double precision, intent(in):: m0,m1,m2

	OmegaLR_12 = 8.d0*m1*m2*(Ibrems_1(m0,m1,m2)+Ibrems_2(m0,m1,m2)&
		-(m0**2-m1**2-m2**2)*Ibrems_12(m0,m1,m2))
	
	end function
!====== Real photon emmision for h -> W W* (124.d0 <= mh <= 126.d0) ======
	double precision function decay_hWcsgam_sm()
	double precision,allocatable:: dat(:,:)
	integer:: num_dat,i

	if (124.d0 <= mh .or. mh <= 126.d0) then
		open(100, file='./modules/dat/decay_hWcsgam_sm.dat')
		read(100, *) !skip first line
		num_dat = 21
		allocate(dat(2,num_dat))

		do i = 1, num_dat
			read(100, *) dat(:,i)
		end do

		decay_hWcsgam_sm = linear_interpolation(dat(1,:),dat(2,:),num_dat,mh)

		deallocate(dat)
		close(100)
	endif

	end function
!-------------------------------------------------------------
	double precision function decay_hWmunugam_sm()
	double precision,allocatable:: dat(:,:)
	integer:: num_dat,i

	if (124.d0 <= mh .or. mh <= 126.d0) then
		open(100, file='./modules/dat/decay_hWmunugam_sm.dat')
		read(100, *) !skip first line
		num_dat = 21
		allocate(dat(2,num_dat))

		do i = 1, num_dat
			read(100, *) dat(:,i)
		end do

		decay_hWmunugam_sm = linear_interpolation(dat(1,:),dat(2,:),num_dat,mh)

		deallocate(dat)
		close(100)
	endif

	end function

end module real_photon_emission